[package]
name = "{{ package_name }}"
version = "0.1.0"
edition = "2021"

[dependencies]
rosidl-runtime-rs = { path = "../rosidl-runtime-rs" }
serde = { version = "1.0", features = ["derive"], optional = true }
{% if needs_big_array %}
serde-big-array = { version = "0.5", optional = true }
{% endif %}
{% for dep in dependencies %}
{{ dep }} = { path = "../{{ dep }}" }
{% endfor %}

[features]
default = []
serde = ["dep:serde"{% if needs_big_array %}, "dep:serde-big-array"{% endif %}]

[lib]
crate-type = ["lib", "cdylib"]
